# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/core/tensor/party.proto
"""Generated protocol buffer code."""
# third party
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1dproto/core/tensor/party.proto\x12\x10syft.core.tensor""\n\x05Party\x12\x0b\n\x03url\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\rb\x06proto3'
)


_PARTY = DESCRIPTOR.message_types_by_name["Party"]
Party = _reflection.GeneratedProtocolMessageType(
    "Party",
    (_message.Message,),
    {
        "DESCRIPTOR": _PARTY,
        "__module__": "proto.core.tensor.party_pb2"
        # @@protoc_insertion_point(class_scope:syft.core.tensor.Party)
    },
)
_sym_db.RegisterMessage(Party)

if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _PARTY._serialized_start = 51
    _PARTY._serialized_end = 85
# @@protoc_insertion_point(module_scope)
