# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/core/node/domain/service/get_all_requests_message.proto
"""Generated protocol buffer code."""
# third party
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


# syft absolute
from syft.proto.core.common import (
    common_object_pb2 as proto_dot_core_dot_common_dot_common__object__pb2,
)
from syft.proto.core.io import address_pb2 as proto_dot_core_dot_io_dot_address__pb2
from syft.proto.core.node.domain.service import (
    request_message_pb2 as proto_dot_core_dot_node_dot_domain_dot_service_dot_request__message__pb2,
)

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n=proto/core/node/domain/service/get_all_requests_message.proto\x12\x1dsyft.core.node.domain.service\x1a\x34proto/core/node/domain/service/request_message.proto\x1a%proto/core/common/common_object.proto\x1a\x1bproto/core/io/address.proto"\x8f\x01\n\x15GetAllRequestsMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12\'\n\x08reply_to\x18\x03 \x01(\x0b\x32\x15.syft.core.io.Address"\xaf\x01\n\x1dGetAllRequestsResponseMessage\x12%\n\x06msg_id\x18\x01 \x01(\x0b\x32\x15.syft.core.common.UID\x12&\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x15.syft.core.io.Address\x12?\n\x08requests\x18\x03 \x03(\x0b\x32-.syft.core.node.domain.service.RequestMessageb\x06proto3'
)


_GETALLREQUESTSMESSAGE = DESCRIPTOR.message_types_by_name["GetAllRequestsMessage"]
_GETALLREQUESTSRESPONSEMESSAGE = DESCRIPTOR.message_types_by_name[
    "GetAllRequestsResponseMessage"
]
GetAllRequestsMessage = _reflection.GeneratedProtocolMessageType(
    "GetAllRequestsMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETALLREQUESTSMESSAGE,
        "__module__": "proto.core.node.domain.service.get_all_requests_message_pb2"
        # @@protoc_insertion_point(class_scope:syft.core.node.domain.service.GetAllRequestsMessage)
    },
)
_sym_db.RegisterMessage(GetAllRequestsMessage)

GetAllRequestsResponseMessage = _reflection.GeneratedProtocolMessageType(
    "GetAllRequestsResponseMessage",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETALLREQUESTSRESPONSEMESSAGE,
        "__module__": "proto.core.node.domain.service.get_all_requests_message_pb2"
        # @@protoc_insertion_point(class_scope:syft.core.node.domain.service.GetAllRequestsResponseMessage)
    },
)
_sym_db.RegisterMessage(GetAllRequestsResponseMessage)

if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _GETALLREQUESTSMESSAGE._serialized_start = 219
    _GETALLREQUESTSMESSAGE._serialized_end = 362
    _GETALLREQUESTSRESPONSEMESSAGE._serialized_start = 365
    _GETALLREQUESTSRESPONSEMESSAGE._serialized_end = 540
# @@protoc_insertion_point(module_scope)
